package com.scbfkj.uni.system;

import com.scbfkj.uni.library.DataEncryptionUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.service.ControlService;
import com.scbfkj.uni.service.LoggerService;
import jakarta.annotation.PostConstruct;
import org.flywaydb.core.Flyway;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.URL;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.Objects;
import java.util.Scanner;


@Component
public class SystemInit {


    @Value("${db.center.config}")
    private String centerConfig;

    @Value("${db.security.config}")
    private String securityConfig;

    @Value("${app.security.enable:true}")
    private boolean securityEnable = false;
    @Value("${app.container.code}")
    private String containerCode;


    @Value("${app.debug:false}")
    private boolean debug = true;

    @PostConstruct
    public void init() throws Exception {
        Config.containerCode = containerCode;
        Config.securityEnable = securityEnable;
        Config.debug = debug;
        Config.centerConnectionStr = DataEncryptionUtil.decryptRSAByPrivateKey(centerConfig);
        Config.securityConnectionStr = DataEncryptionUtil.decryptRSAByPrivateKey(securityConfig);
        migrateDB();

//        日志配置初始化
        Config.targets.addAll(DataBase.query(Config.centerConnectionStr, "select * from systeminfo where 1=?", Collections.singletonList(new Object[]{1})));

        initializeSystemEnvironment();
        ControlService.startServiceByContainerCode();
//        日志任务
        ScheduleUtil.startByService(LoggerService::sendMessage, 100);
//        LoggerService.logSystemError(LocalDateTime.now(), "", "");
        LoggerService.log(LoggerService.LogType.SYSTEM, new HashMap<>() {{
            put("occurrencetime", LocalDateTime.now());
        }});
    }

    private void initializeSystemEnvironment() throws Exception {
//        运行环境初始化
        File resource = new File("logs");
        boolean exists = resource.exists();
        if (!exists) {
            if (!resource.mkdirs()) {
                throw new RuntimeException("创建logs目录失败");
            } else {
                System.out.println("创建目录成功:" + resource.getAbsolutePath());
            }
        }
//        运行环境初始化
        resource = new File("plugins");
        exists = resource.exists();
        if (!exists) {
            if (!resource.mkdirs()) {
                throw new RuntimeException("创建plugins目录失败");
            } else {
                System.out.println("创建目录成功:" + resource.getAbsolutePath());
            }
        }
        readConfigFile();
        cleanRemoteConfig();
        ControlService.startServiceByContainerCode();
    }

    private String readConfigFile() throws Exception {
        File resource = new File("config");
        if (resource.exists()) {
            BufferedInputStream inputStream = new BufferedInputStream(new FileInputStream(resource));
            byte[] bytes = new byte[1024];
            StringBuilder builder = new StringBuilder();
            while (inputStream.read(bytes) > 0) {
                builder.append(new String(bytes));
            }
            String string = builder.toString();
            return DataEncryptionUtil.decryptRSAByPrivateKey(string);
        } else {
            return "{}";
        }
    }

    /**
     * 清理远程数据配置
     */
    private void cleanRemoteConfig() throws Exception {
        URL url = ResourceUtils.getURL("classpath:sql");
        File file = new File(url.getPath());
        removeRemoteData();
        if (file.exists()) {
            if (!file.isFile()) {
                for (File listFile : file.listFiles()) {
                    String sqls = readFileToString(listFile);
                    for (String sql : sqls.split(";")) {
                        if (!sql.trim().isEmpty()) {
                            DataBase.baseUpdate(Config.centerConnectionStr, sql);
                        }
                    }
                }
            }
        }
    }

    /**
     * 删除远程数据库
     */
    private void removeRemoteData() {

    }

    public static String readFileToString(File file) {
        StringBuilder content = new StringBuilder();
        try (Scanner scanner = new Scanner(file)) {
            while (scanner.hasNextLine()) {
                content.append(scanner.nextLine());
            }
        } catch (FileNotFoundException e) {
            if (Config.debug) {
                e.printStackTrace();
            }
        }
        return content.toString();
    }

    /**
     * 迁移数据，数据库管理
     *
     * @throws Exception
     */
    private void migrateDB() throws Exception {
        Flyway systemSetFlyway = Flyway.configure().dataSource(DataBase.getDataSourcePool(Config.centerConnectionStr).getUnwrappedDataSource()).locations("db/systemset").baselineOnMigrate(true).load();
        systemSetFlyway.migrate();
        Flyway uniAuthFlyway = Flyway.configure().dataSource(DataBase.getDataSourcePool(Config.securityConnectionStr).getUnwrappedDataSource()).locations("db/uniauth").baselineOnMigrate(true).load();
        uniAuthFlyway.migrate();
    }
}
